package betterwithmods.common.registry;

import betterwithmods.common.blocks.mechanical.tile.TileEntityFilteredHopper;
import betterwithmods.util.InvUtils;
import com.google.common.collect.Lists;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.items.ItemStackHandler;

import java.util.ArrayList;
import java.util.List;

/**
 * Purpose:
 *
 * @author Tyler Marshall
 * @version 11/13/16
 */
public class HopperInteractions {
    public static final ArrayList<HopperRecipe> RECIPES = new ArrayList<>();

    public static void addHopperRecipe(HopperRecipe recipe) {
        RECIPES.add(recipe);
    }

    static {

    }

    public static boolean attemptToCraft(int filterType, World world, BlockPos pos, EntityItem input) {
        for (HopperRecipe recipe : RECIPES) {
            if (recipe.isRecipe(filterType, input)) {
                if (recipe.canCraft(world, pos)) {
                    recipe.craft(input, world, pos);
                    return true;
                }
            }
        }
        return false;
    }

    public static class SoulUrnRecipe extends HopperRecipe {
        public SoulUrnRecipe(ItemStack input, ItemStack output, ItemStack... created) {
            super(6, input, output, created);
        }

        @Override
        public void craft(EntityItem inputStack, World world, BlockPos pos) {
            InvUtils.ejectStackWithOffset(world, inputStack.func_180425_c(), output.func_77946_l());
            onCraft(world, pos, inputStack);
        }

        @Override
        public void onCraft(World world, BlockPos pos, EntityItem item) {
            ((TileEntityFilteredHopper) world.func_175625_s(pos)).increaseSoulCount(1);
            if (!world.field_72995_K) {
                world.func_184133_a(null, pos, SoundEvents.field_187551_bH, SoundCategory.BLOCKS, 1.0F, world.field_73012_v.nextFloat() * 0.1F + 0.45F);
            }
            super.onCraft(world, pos, item);
        }

        @Override
        public boolean canCraft(World world, BlockPos pos) {
            return true;
        }

        public ItemStack getInput() {
            ItemStack i = input.func_77946_l();
            if(!secondaryOutput.isEmpty())
                i.func_190920_e(8);
            return i;
        }

        public ItemStack getOutput() {
            ItemStack o = output.func_77946_l();
            if(!secondaryOutput.isEmpty())
                o.func_190920_e(8);
            return o;
        }

    }

    public static abstract class HopperRecipe {
        public final ItemStack input;
        public final ItemStack output;
        public List<ItemStack> secondaryOutput = Lists.newArrayList();
        private final int filterType;

        public HopperRecipe(int filterType, ItemStack input, ItemStack output, ItemStack... secondaryOutput) {
            this.filterType = filterType;
            this.input = input;
            this.output = output;
            if (secondaryOutput != null)
                this.secondaryOutput = Lists.newArrayList(secondaryOutput);
        }

        public boolean isRecipe(int filterType, EntityItem inputStack) {
            if (filterType == this.filterType) {
                if (inputStack != null) {
                    ItemStack i = inputStack.func_92059_d();
                    return InvUtils.matches(i, input);
                }
                return false;
            }
            return false;
        }

        public void craft(EntityItem inputStack, World world, BlockPos pos) {
            InvUtils.ejectStackWithOffset(world, inputStack.func_180425_c(), output.func_77946_l());
            for (int i = 0; i < inputStack.func_92059_d().func_190916_E(); i++)
                InvUtils.ejectStackWithOffset(world, inputStack.func_180425_c(), secondaryOutput);
            onCraft(world, pos, inputStack);
        }

        public void onCraft(World world, BlockPos pos, EntityItem item) {
            item.func_92059_d().func_190918_g(1);
            if (item.func_92059_d().func_190916_E() <= 0)
                item.func_70106_y();
        }

        public int getFilterType() {
            return filterType;
        }

        public ItemStack getInput() {
            return input;
        }

        public ItemStack getOutput() {
            return output;
        }

        public List<ItemStack> getSecondaryOutput() {
            return secondaryOutput;
        }

        public boolean canCraft(World world, BlockPos pos) {
            TileEntityFilteredHopper tile = (TileEntityFilteredHopper) world.func_175625_s(pos);
            ItemStackHandler inventory = tile.inventory;
            boolean flag = true;
            if (!secondaryOutput.isEmpty()) {
                for (ItemStack stack : secondaryOutput) {
                    if (!InvUtils.insert(inventory, stack, true).func_190926_b()) {
                        flag = false;
                        break;
                    }
                }
            }
            return flag;
        }
    }
}
